import { PrivateValueStore, NavigationRouteContext, NavigationContext, } from "@react-navigation/native";
import { CommonActions, } from "@react-navigation/routers";
import * as React from "react";
import { isValidElementType } from "react-is";
import { useCustomOnAction } from "./useCustomOnAction";
import { useDescriptors } from "./useDescriptors";
// This is to make TypeScript compiler happy
// eslint-disable-next-line babel/no-unused-expressions
PrivateValueStore;
/**
 * Extract route config object from React children elements.
 *
 * @param children React Elements to extract the config from.
 */
const getRouteConfigsFromChildren = (children, groupKey, groupOptions) => {
    const configs = React.Children.toArray(children).reduce((acc, child) => {
        if (React.isValidElement(child)) {
            acc.push({
                keys: [groupKey, child.props.navigationKey],
                options: groupOptions,
                props: child.props,
            });
            return acc;
        }
        throw new Error(`A navigator can only contain 'Screen', 'Group' or 'React.Fragment' as its direct children (found ${React.isValidElement(child)
            ? `'${typeof child.type === "string" ? child.type : child.type?.name}'${child.props != null &&
                typeof child.props === "object" &&
                "name" in child.props &&
                child.props?.name
                ? ` for the screen '${child.props.name}'`
                : ""}`
            : typeof child === "object"
                ? JSON.stringify(child)
                : `'${String(child)}'`}). To render this component in the navigator, pass it in the 'component' prop to 'Screen'.`);
    }, []);
    if (process.env.NODE_ENV !== "production") {
        configs.forEach((config) => {
            const { name, children, component, getComponent } = config.props;
            if (typeof name !== "string" || !name) {
                throw new Error(`Got an invalid name (${JSON.stringify(name)}) for the screen. It must be a non-empty string.`);
            }
            if (children != null ||
                component !== undefined ||
                getComponent !== undefined) {
                if (children != null && component !== undefined) {
                    throw new Error(`Got both 'component' and 'children' props for the screen '${name}'. You must pass only one of them.`);
                }
                if (children != null && getComponent !== undefined) {
                    throw new Error(`Got both 'getComponent' and 'children' props for the screen '${name}'. You must pass only one of them.`);
                }
                if (component !== undefined && getComponent !== undefined) {
                    throw new Error(`Got both 'component' and 'getComponent' props for the screen '${name}'. You must pass only one of them.`);
                }
                if (children != null && typeof children !== "function") {
                    throw new Error(`Got an invalid value for 'children' prop for the screen '${name}'. It must be a function returning a React Element.`);
                }
                if (component !== undefined && !isValidElementType(component)) {
                    throw new Error(`Got an invalid value for 'component' prop for the screen '${name}'. It must be a valid React Component.`);
                }
                if (getComponent !== undefined && typeof getComponent !== "function") {
                    throw new Error(`Got an invalid value for 'getComponent' prop for the screen '${name}'. It must be a function returning a React Component.`);
                }
                if (typeof component === "function") {
                    if (component.name === "component") {
                        // Inline anonymous functions passed in the `component` prop will have the name of the prop
                        // It's relatively safe to assume that it's not a component since it should also have PascalCase name
                        // We won't catch all scenarios here, but this should catch a good chunk of incorrect use.
                        console.warn(`Looks like you're passing an inline function for 'component' prop for the screen '${name}' (e.g. component={() => <SomeComponent />}). Passing an inline function will cause the component state to be lost on re-render and cause perf issues since it's re-created every render. You can pass the function as children to 'Screen' instead to achieve the desired behaviour.`);
                    }
                    else if (/^[a-z]/.test(component.name)) {
                        console.warn(`Got a component with the name '${component.name}' for the screen '${name}'. React Components must start with an uppercase letter. If you're passing a regular function and not a component, pass it as children to 'Screen' instead. Otherwise capitalize your component's name.`);
                    }
                }
            }
            else {
                throw new Error(`Couldn't find a 'component', 'getComponent' or 'children' prop for the screen '${name}'. This can happen if you passed 'undefined'. You likely forgot to export your component from the file it's defined in, or mixed up default import and named import when importing.`);
            }
        });
    }
    return configs;
};
/**
 * Hook for building navigators.
 *
 * @param createRouter Factory method which returns router object.
 * @param options Options object containing `children` and additional options for the router.
 * @returns An object containing `state`, `navigation`, `descriptors` objects.
 */
export default function useNavigationBuilder(createRouter, options) {
    const route = React.useContext(NavigationRouteContext);
    const { children, screenListeners, ...rest } = options;
    const { current: router } = React.useRef(createRouter({
        ...rest,
    }));
    const routeConfigs = getRouteConfigsFromChildren(children);
    const screens = routeConfigs.reduce((acc, config) => {
        if (config.props.name in acc) {
            throw new Error(`A navigator cannot contain multiple 'Screen' components with the same name (found duplicate screen named '${config.props.name}')`);
        }
        acc[config.props.name] = config;
        return acc;
    }, {});
    const routeNames = routeConfigs.map((config) => config.props.name);
    const routeParamList = routeNames.reduce((acc, curr) => {
        const { initialParams } = screens[curr].props;
        acc[curr] = initialParams;
        return acc;
    }, {});
    const routeGetIdList = routeNames.reduce((acc, curr) => Object.assign(acc, {
        [curr]: screens[curr].props.getId,
    }), {});
    if (!routeNames.length) {
        throw new Error("Couldn't find any screens for the navigator. Have you defined any screens as its children?");
    }
    const { state: currentState, getState: getCurrentState, setState: setCurrentState, } = React.useContext(NavigationStateContext);
    const stateCleanedUp = React.useRef(false);
    const setState = React.useCallback((state) => {
        if (stateCleanedUp.current) {
            // State might have been already cleaned up due to unmount
            // We do not want to expose API allowing to override this
            // This would lead to old data preservation on main navigator unmount
            return;
        }
        setCurrentState(state);
    }, [setCurrentState]);
    const [initializedState, isFirstStateInitialization] = React.useMemo(() => {
        const initialRouteParamList = routeNames.reduce((acc, curr) => {
            // les params de la route A ajouter dans un second temps
            const { initialParams } = screens[curr].props;
            const initialParamsFromParams = route?.params?.state == null &&
                route?.params?.initial !== false &&
                route?.params?.screen === curr
                ? route.params.params
                : undefined;
            acc[curr] =
                initialParams !== undefined || initialParamsFromParams !== undefined
                    ? {
                        ...initialParams,
                        ...initialParamsFromParams,
                    }
                    : undefined;
            return acc;
        }, {});
        return [
            router.getInitialState({
                routeNames,
                routeParamList: initialRouteParamList,
                routeGetIdList,
            }),
            true,
        ];
        // We explicitly don't include routeNames, route.params etc. in the dep list
        // below. We want to avoid forcing a new state to be calculated in those cases
        // Instead, we handle changes to these in the nextState code below. Note
        // that some changes to routeConfigs are explicitly ignored, such as changes
        // to initialParams
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [currentState, router]);
    let state = 
    // If the state isn't initialized, or stale, use the state we initialized instead
    // The state won't update until there's a change needed in the state we have initalized locally
    // So it'll be `undefined` or stale until the first navigation event happens
    currentState !== undefined
        ? currentState
        : initializedState;
    const initializedStateRef = React.useRef();
    initializedStateRef.current = initializedState; // Je comprends pas pourquoi cette ref, pourquoi ne pas directement récupérer
    const getState = React.useCallback(() => {
        const currentState = getCurrentState();
        return currentState !== undefined
            ? currentState
            : initializedStateRef.current;
    }, [getCurrentState]);
    const onAction = useCustomOnAction({
        router,
        getState,
        setState,
        key: route?.key, // a ajouter dans un second temps
        routerConfigOptions: {
            routeNames,
            routeParamList,
            routeGetIdList,
        },
    });
    const navigation = useNavigationHelpers({
        id: options.id,
        onAction,
        getState,
        router,
    });
    const { descriptors } = useDescriptors({
        state,
        screens,
        navigation,
        screenOptions: options.screenOptions,
        onAction,
        getState,
        setState,
        router,
        // @ts-expect-error: this should have both core and custom events, but too much work right now
    });
    const NavigationContent = useComponent((children) => (<>
      {children}
    </>));
    return {
        state,
        navigation,
        descriptors,
        NavigationContent,
    };
}
const NavigationContent = ({ render, children }) => {
    return render(children);
};
export function useComponent(render) {
    const renderRef = React.useRef(render);
    // Normally refs shouldn't be mutated in render
    // But we return a component which will be rendered
    // So it's just for immediate consumption
    renderRef.current = render;
    React.useEffect(() => {
        renderRef.current = null;
    });
    return React.useRef(({ children }) => {
        const render = renderRef.current;
        if (render === null) {
            throw new Error("The returned component must be rendered in the same render phase as the hook.");
        }
        return <NavigationContent render={render}>{children}</NavigationContent>;
    }).current;
}
const MISSING_CONTEXT_ERROR = "Couldn't find a navigation context. Have you wrapped your app with 'NavigationContainer'? See https://reactnavigation.org/docs/getting-started for setup instructions.";
export const NavigationStateContext = React.createContext({
    isDefault: true,
    get getState() {
        throw new Error(MISSING_CONTEXT_ERROR);
    },
    get setState() {
        throw new Error(MISSING_CONTEXT_ERROR);
    },
});
/**
 * Navigation object with helper methods to be used by a navigator.
 * This object includes methods for common actions as well as methods the parent screen's navigation object.
 */
export function useNavigationHelpers({ id: navigatorId, onAction, getState, router }) {
    const parentNavigationHelpers = React.useContext(NavigationContext);
    return React.useMemo(() => {
        const dispatch = (op) => {
            const action = typeof op === "function" ? op(getState()) : op;
            onAction(action);
        };
        const actions = {
            ...router.actionCreators,
            ...CommonActions,
        };
        const helpers = Object.keys(actions).reduce((acc, name) => {
            // @ts-expect-error: name is a valid key, but TypeScript is dumb
            acc[name] = (...args) => dispatch(actions[name](...args));
            return acc;
        }, {});
        const navigationHelpers = {
            ...parentNavigationHelpers,
            ...helpers,
            dispatch,
            isFocused: parentNavigationHelpers
                ? parentNavigationHelpers.isFocused
                : () => true,
            canGoBack: () => {
                const state = getState();
                return (router.getStateForAction(state, CommonActions.goBack(), {
                    routeNames: state.routeNames,
                    routeParamList: {},
                    routeGetIdList: {},
                }) !== null ||
                    parentNavigationHelpers?.canGoBack() ||
                    false);
            },
            getId: () => navigatorId,
            getParent: (id) => {
                if (id !== undefined) {
                    let current = navigationHelpers;
                    while (current && id !== current.getId()) {
                        current = current.getParent();
                    }
                    return current;
                }
                return parentNavigationHelpers;
            },
            getState,
        };
        return navigationHelpers;
    }, [navigatorId, getState, parentNavigationHelpers, router]);
}
