import * as React from "react";
import { NavigationBuilderContext, } from "./NavigationBuilderContext";
import { NavigationContext } from "./NavigationContext";
import { NavigationRouteContext, } from "@react-navigation/native";
import { useNavigationCache } from "./useNavigationCache";
import { useRouteCache } from "./useRouteCache";
import { NavigationStateContext } from "./useCustomNavigationBuilder";
/**
 * Hook to create descriptor objects for the child routes.
 *
 * A descriptor object provides 3 things:
 * - Helper method to render a screen
 * - Options specified by the screen for the navigator
 * - Navigation object intended for the route
 */
export function useDescriptors({ state, screens, navigation, screenOptions, onAction, getState, setState, router, }) {
    const [options, setOptions] = React.useState({});
    const context = React.useMemo(() => ({
        navigation,
        onAction,
    }), [navigation, onAction, ,]);
    const { navigations } = useNavigationCache({
        state,
        getState,
        navigation,
        setOptions,
        router,
    });
    const routes = useRouteCache(state.routes);
    const getOptions = (route, navigation, overrides) => {
        const config = screens[route.name];
        const screen = config.props;
        const optionsList = [
            // The default `screenOptions` passed to the navigator
            screenOptions,
            // The `screenOptions` props passed to `Group` elements
            ...(config.options
                ? config.options.filter(Boolean)
                : []),
            // The `options` prop passed to `Screen` elements,
            screen.options,
            // The options set via `navigation.setOptions`
            overrides,
        ];
        return optionsList.reduce((acc, curr) => Object.assign(acc, 
        // @ts-expect-error: we check for function but TS still complains
        typeof curr !== "function" ? curr : curr({ route, navigation, theme })), {});
    };
    const render = (route, navigation, customOptions, routeState) => {
        const config = screens[route.name];
        const screen = config.props;
        const clearOptions = () => setOptions((o) => {
            if (route.key in o) {
                // eslint-disable-next-line @typescript-eslint/no-unused-vars
                const { [route.key]: _, ...rest } = o;
                return rest;
            }
            return o;
        });
        let element = (<SceneView navigation={navigation} route={route} screen={screen} routeState={routeState} getState={getState} setState={setState} options={customOptions} clearOptions={clearOptions}/>);
        return (<NavigationBuilderContext.Provider key={route.key} value={context}>
        <NavigationContext.Provider value={navigation}>
          <NavigationRouteContext.Provider value={route}>
            {element}
          </NavigationRouteContext.Provider>
        </NavigationContext.Provider>
      </NavigationBuilderContext.Provider>);
    };
    const descriptors = routes.reduce((acc, route, i) => {
        const navigation = navigations[route.key];
        const customOptions = getOptions(route, navigation, options[route.key]);
        const element = render(route, navigation, customOptions, state.routes[i].state);
        acc[route.key] = {
            route,
            // @ts-expect-error: it's missing action helpers, fix later
            navigation,
            render() {
                return element;
            },
            options: customOptions,
        };
        return acc;
    }, {});
    return {
        descriptors,
    };
}
/**
 * Component which takes care of rendering the screen for a route.
 * It provides all required contexts and applies optimizations when applicable.
 */
export function SceneView({ screen, route, navigation, routeState, getState, setState, options, clearOptions, }) {
    const getCurrentState = React.useCallback(() => {
        const state = getState();
        const currentRoute = state.routes.find((r) => r.key === route.key);
        return currentRoute ? currentRoute.state : undefined;
    }, [getState, route.key]);
    const setCurrentState = React.useCallback((child) => {
        const state = getState();
        setState({
            ...state,
            routes: state.routes.map((r) => r.key === route.key ? { ...r, state: child } : r),
        });
    }, [getState, route.key, setState]);
    const isInitialRef = React.useRef(true);
    React.useEffect(() => {
        isInitialRef.current = false;
    });
    // Clear options set by this screen when it is unmounted
    React.useEffect(() => {
        return clearOptions;
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    const context = React.useMemo(() => ({
        state: routeState,
        getState: getCurrentState,
        setState: setCurrentState,
    }), [routeState, getCurrentState, setCurrentState]);
    const ScreenComponent = screen.getComponent
        ? screen.getComponent()
        : screen.component;
    return (<NavigationStateContext.Provider value={context}>
      {ScreenComponent !== undefined ? (<ScreenComponent navigation={navigation} route={route}/>) : screen.children !== undefined ? (screen.children({ navigation, route })) : null}
    </NavigationStateContext.Provider>);
}
